Title: Canvas Image
Order: 70
---

To add [ImageSharp](https://github.com/SixLabors/ImageSharp) superpowers to 
your console application to draw images, you will need to install 
the [Spectre.Console.ImageSharp](https://www.nuget.org/packages/Spectre.Console.ImageSharp) NuGet package.

```text
> dotnet add package Spectre.Console.ImageSharp
```

# Loading images

Once you've added the `Spectre.Console.ImageSharp` NuGet package, 
you can create a new instance of `CanvasImage` to draw images to the console.

```csharp
// Load an image
var image = new CanvasImage("cake.png");

// Set the max width of the image.
// If no max width is set, the image will take
// up as much space as there is available.
image.MaxWidth(16);

// Render the image to the console
AnsiConsole.Render(image);
```

## Result

<pre style="font-size:90%;font-family:consolas,'Courier New',monospace;line-height: normal; padding: 0px;background-color: #222222; padding: 20px;">
<span>        </span><span style="background-color: #542813">  </span><span style="background-color: #572F1B">  </span><span style="background-color: #4E1F09">  </span><span style="background-color: #5B3826">  </span><span style="background-color: #5E3A29">  </span><span style="background-color: #532611">  </span><span>            </span>
<span>        </span><span style="background-color: #562E1B">  </span><span style="background-color: #634737">  </span><span style="background-color: #562E1A">  </span><span style="background-color: #5D4132">  </span><span style="background-color: #6D584B">  </span><span style="background-color: #624332">  </span><span style="background-color: #562B17">  </span><span>          </span>
<span>        </span><span style="background-color: #512714">  </span><span style="background-color: #654E40">  </span><span style="background-color: #705243">  </span><span style="background-color: #745749">  </span><span style="background-color: #6D5B4F">  </span><span style="background-color: #715E52">  </span><span style="background-color: #644636">  </span><span style="background-color: #6A4433">  </span><span style="background-color: #542916">  </span><span style="background-color: #431C0B">  </span><span>    </span>
<span>      </span><span style="background-color: #491E0A">  </span><span style="background-color: #5C3523">  </span><span style="background-color: #695346">  </span><span style="background-color: #705C4F">  </span><span style="background-color: #654838">  </span><span style="background-color: #654A3A">  </span><span style="background-color: #726154">  </span><span style="background-color: #715D50">  </span><span style="background-color: #B8A79F">  </span><span style="background-color: #AE988F">  </span><span style="background-color: #6F4A39">  </span><span style="background-color: #441906">  </span><span>  </span>
<span>    </span><span style="background-color: #532916">  </span><span style="background-color: #8A6C5E">  </span><span style="background-color: #C2B3AB">  </span><span style="background-color: #8B786E">  </span><span style="background-color: #6B584C">  </span><span style="background-color: #695143">  </span><span style="background-color: #6C5648">  </span><span style="background-color: #6F5D51">  </span><span style="background-color: #816A55">  </span><span style="background-color: #E7E1DA">  </span><span style="background-color: #F9F5EE">  </span><span style="background-color: #BAA593">  </span><span style="background-color: #61381F">  </span><span>  </span>
<span style="background-color: #421C0A">  </span><span style="background-color: #603826">  </span><span style="background-color: #9E8479">  </span><span style="background-color: #E2DAD6">  </span><span style="background-color: #FBF9F6">  </span><span style="background-color: #F0EADF">  </span><span style="background-color: #C4B59D">  </span><span style="background-color: #9D8663">  </span><span style="background-color: #786451">  </span><span style="background-color: #705D4E">  </span><span style="background-color: #BFA052">  </span><span style="background-color: #FEE88B">  </span><span style="background-color: #FDE580">  </span><span style="background-color: #E2C362">  </span><span style="background-color: #794E1D">  </span><span>  </span>
<span style="background-color: #4B1D05">  </span><span style="background-color: #A6844C">  </span><span style="background-color: #E9D595">  </span><span style="background-color: #F1DC92">  </span><span style="background-color: #F5DD83">  </span><span style="background-color: #FBE278">  </span><span style="background-color: #FFE36E">  </span><span style="background-color: #F1D25E">  </span><span style="background-color: #866F4B">  </span><span style="background-color: #726256">  </span><span style="background-color: #967945">  </span><span style="background-color: #F5D456">  </span><span style="background-color: #F8D756">  </span><span style="background-color: #E1BE4A">  </span><span style="background-color: #7D511B">  </span><span>  </span>
<span style="background-color: #4F2005">  </span><span style="background-color: #C9A441">  </span><span style="background-color: #FFE05C">    </span><span style="background-color: #FEDF5B">  </span><span style="background-color: #FCDC59">  </span><span style="background-color: #F7D555">  </span><span style="background-color: #E5C04A">  </span><span style="background-color: #795E3B">  </span><span style="background-color: #726256">  </span><span style="background-color: #755F4C">  </span><span style="background-color: #A17124">  </span><span style="background-color: #AE7414">  </span><span style="background-color: #AE791D">  </span><span style="background-color: #794D18">  </span><span>  </span>
<span style="background-color: #4E1F04">  </span><span style="background-color: #B78D31">  </span><span style="background-color: #DDB33E">  </span><span style="background-color: #D0A132">  </span><span style="background-color: #C28F25">  </span><span style="background-color: #B67E1A">  </span><span style="background-color: #AC7111">  </span><span style="background-color: #9E610A">  </span><span style="background-color: #5F3212">  </span><span style="background-color: #6A574B">  </span><span style="background-color: #726256">  </span><span style="background-color: #744D2A">  </span><span style="background-color: #955401">  </span><span style="background-color: #8C5106">  </span><span style="background-color: #5F310C">  </span><span>  </span>
<span style="background-color: #4B1A00">  </span><span style="background-color: #854903">  </span><span style="background-color: #9B5A02">  </span><span style="background-color: #995700">        </span><span style="background-color: #935200">  </span><span style="background-color: #592402">  </span><span style="background-color: #5B3F30">  </span><span style="background-color: #726256">  </span><span style="background-color: #705A4A">  </span><span style="background-color: #844C0C">  </span><span style="background-color: #824400">  </span><span style="background-color: #4C1B00">  </span><span>  </span>
<span style="background-color: #4B1A00">  </span><span style="background-color: #824500">  </span><span style="background-color: #995700">          </span><span style="background-color: #935200">  </span><span style="background-color: #592300">  </span><span style="background-color: #4F2411">  </span><span style="background-color: #6B584C">  </span><span style="background-color: #736256">  </span><span style="background-color: #734E2C">  </span><span style="background-color: #7C4101">  </span><span style="background-color: #4C1B00">  </span><span>  </span>
<span style="background-color: #4B1A00">  </span><span style="background-color: #824500">  </span><span style="background-color: #995700">          </span><span style="background-color: #935200">  </span><span style="background-color: #592300">  </span><span style="background-color: #4A1902">  </span><span style="background-color: #5C4031">  </span><span style="background-color: #726256">  </span><span style="background-color: #705B4B">  </span><span style="background-color: #6A390F">  </span><span style="background-color: #4C1A00">  </span><span>  </span>
<span style="background-color: #4B1A00">  </span><span style="background-color: #824500">  </span><span style="background-color: #995700">          </span><span style="background-color: #935200">  </span><span style="background-color: #592300">  </span><span style="background-color: #4A1700">  </span><span style="background-color: #4F2512">  </span><span style="background-color: #6B594D">  </span><span style="background-color: #736256">  </span><span style="background-color: #634432">  </span><span style="background-color: #4C1D08">  </span><span>  </span>
<span style="background-color: #4B1A00">  </span><span style="background-color: #814400">  </span><span style="background-color: #955400">  </span><span style="background-color: #915100">  </span><span style="background-color: #8C4D00">  </span><span style="background-color: #864800">  </span><span style="background-color: #7F4301">  </span><span style="background-color: #743A01">  </span><span style="background-color: #521E01">  </span><span style="background-color: #4A1700">  </span><span style="background-color: #4A1902">  </span><span style="background-color: #5D4132">  </span><span style="background-color: #726256">  </span><span style="background-color: #6F5B4E">  </span><span style="background-color: #5D3A28">  </span><span style="background-color: #53220C">  </span>
<span style="background-color: #471801">  </span><span style="background-color: #642D01">  </span><span style="background-color: #6B3301">  </span><span style="background-color: #642E02">  </span><span style="background-color: #5D2902">  </span><span style="background-color: #542203">  </span><span style="background-color: #4C1C04">  </span><span style="background-color: #461905">  </span><span style="background-color: #4A1C07">  </span><span style="background-color: #4C1A03">  </span><span style="background-color: #4B1801">  </span><span style="background-color: #502613">  </span><span style="background-color: #69564A">  </span><span style="background-color: #705F54">  </span><span style="background-color: #604232">  </span><span style="background-color: #51200A">  </span>
<span style="background-color: #411806">  </span><span style="background-color: #431A07">  </span><span style="background-color: #411D0D">  </span><span>              </span><span style="background-color: #4D1B05">  </span><span style="background-color: #4D1D07">  </span><span style="background-color: #533324">  </span><span style="background-color: #583E30">  </span><span style="background-color: #53301F">  </span><span style="background-color: #53230D">  </span>
</pre>

# Manipulating images

You can take full advantage of [ImageSharp](https://github.com/SixLabors/ImageSharp)
and manipulate images directly via it's [Processing API](https://docs.sixlabors.com/api/ImageSharp/SixLabors.ImageSharp.Processing.html).

```csharp
// Load an image
var image = new CanvasImage("cake.png");
image.MaxWidth(32);

// Set a sampler that will be used when scaling the image.
image.BilinearResampler();

// Mutate the image using ImageSharp
image.Mutate(ctx => ctx.Grayscale().Rotate(-45).EntropyCrop());

// Render the image to the console
AnsiConsole.Render(image);
```

## Result

<pre style="font-size:90%;font-family:consolas,'Courier New',monospace;line-height: normal; padding: 0px;background-color: #222222; padding: 20px;">
<span>                    </span><span style="background-color: #282828">  </span><span style="background-color: #222222">  </span><span style="background-color: #232323">  </span><span style="background-color: #353535">  </span><span style="background-color: #4B4B4B">  </span><span style="background-color: #595959">    </span><span style="background-color: #3B3B3B">  </span><span style="background-color: #202020">  </span><span style="background-color: #191919">  </span><span>                        </span>
<span>          </span><span style="background-color: #343434">  </span><span style="background-color: #2B2B2B">  </span><span style="background-color: #292929">  </span><span style="background-color: #272727">  </span><span style="background-color: #252525">  </span><span style="background-color: #292929">  </span><span style="background-color: #555555">  </span><span style="background-color: #929292">  </span><span style="background-color: #C7C7C7">  </span><span style="background-color: #E5E5E5">  </span><span style="background-color: #F0F0F0">  </span><span style="background-color: #E4E4E4">  </span><span style="background-color: #A8A8A8">  </span><span style="background-color: #515151">  </span><span style="background-color: #202020">  </span><span style="background-color: #191919">  </span><span>                      </span>
<span>    </span><span style="background-color: #2E2E2E">  </span><span style="background-color: #2B2B2B">  </span><span style="background-color: #333333">  </span><span style="background-color: #373737">  </span><span style="background-color: #3C3C3C">  </span><span style="background-color: #414141">  </span><span style="background-color: #474747">  </span><span style="background-color: #4B4B4B">  </span><span style="background-color: #454545">  </span><span style="background-color: #828282">  </span><span style="background-color: #E0E0E0">  </span><span style="background-color: #FFFFFF">    </span><span style="background-color: #FCFCFC">  </span><span style="background-color: #DEDEDE">  </span><span style="background-color: #DADADA">  </span><span style="background-color: #BCBCBC">  </span><span style="background-color: #515151">  </span><span style="background-color: #202020">  </span><span style="background-color: #191919">  </span><span>                    </span>
<span>    </span><span style="background-color: #272727">  </span><span style="background-color: #414141">  </span><span style="background-color: #5C5C5C">  </span><span style="background-color: #616161">  </span><span style="background-color: #636363">  </span><span style="background-color: #656565">  </span><span style="background-color: #666666">    </span><span style="background-color: #656565">  </span><span style="background-color: #5A5A5A">  </span><span style="background-color: #707070">  </span><span style="background-color: #F3F3F3">  </span><span style="background-color: #FFFFFF">  </span><span style="background-color: #F0F0F0">  </span><span style="background-color: #DDDDDD">      </span><span style="background-color: #BABABA">  </span><span style="background-color: #505050">  </span><span style="background-color: #202020">  </span><span style="background-color: #1B1B1B">  </span><span>                  </span>
<span>    </span><span style="background-color: #242424">  </span><span style="background-color: #3B3B3B">  </span><span style="background-color: #545454">  </span><span style="background-color: #606060">  </span><span style="background-color: #656565">  </span><span style="background-color: #666666">        </span><span style="background-color: #606060">  </span><span style="background-color: #575757">  </span><span style="background-color: #E8E8E8">  </span><span style="background-color: #F6F6F6">  </span><span style="background-color: #E1E1E1">  </span><span style="background-color: #DDDDDD">    </span><span style="background-color: #D9D9D9">  </span><span style="background-color: #A0A0A0">  </span><span style="background-color: #989898">  </span><span style="background-color: #4E4E4E">  </span><span style="background-color: #222222">  </span><span>                  </span>
<span style="background-color: #2F2F2F">  </span><span style="background-color: #2C2C2C">  </span><span style="background-color: #222222">  </span><span style="background-color: #282828">  </span><span style="background-color: #2D2D2D">  </span><span style="background-color: #3E3E3E">  </span><span style="background-color: #4D4D4D">  </span><span style="background-color: #616161">    </span><span style="background-color: #636363">  </span><span style="background-color: #666666">  </span><span style="background-color: #606060">  </span><span style="background-color: #535353">  </span><span style="background-color: #D4D4D4">  </span><span style="background-color: #E2E2E2">  </span><span style="background-color: #DDDDDD">    </span><span style="background-color: #DCDCDC">  </span><span style="background-color: #AFAFAF">  </span><span style="background-color: #666666">  </span><span style="background-color: #6F6F6F">  </span><span style="background-color: #717171">  </span><span style="background-color: #242424">  </span><span style="background-color: #191919">  </span><span>                </span>
<span style="background-color: #2C2C2C">  </span><span style="background-color: #343434">  </span><span style="background-color: #2E2E2E">  </span><span style="background-color: #262626">  </span><span style="background-color: #404040">  </span><span style="background-color: #868686">  </span><span style="background-color: #4D4D4D">  </span><span style="background-color: #5A5A5A">  </span><span style="background-color: #3D3D3D">  </span><span style="background-color: #474747">  </span><span style="background-color: #646464">  </span><span style="background-color: #616161">  </span><span style="background-color: #4D4D4D">  </span><span style="background-color: #9D9D9D">  </span><span style="background-color: #C8C8C8">  </span><span style="background-color: #DADADA">  </span><span style="background-color: #DDDDDD">  </span><span style="background-color: #C4C4C4">  </span><span style="background-color: #717171">  </span><span style="background-color: #5F5F5F">    </span><span style="background-color: #595959">  </span><span style="background-color: #343434">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #191919">  </span><span>              </span>
<span style="background-color: #343434">  </span><span style="background-color: #575757">  </span><span style="background-color: #555555">  </span><span style="background-color: #454545">  </span><span style="background-color: #4C4C4C">  </span><span style="background-color: #656565">  </span><span style="background-color: #5B5B5B">  </span><span style="background-color: #434343">  </span><span style="background-color: #3E3E3E">  </span><span style="background-color: #595959">  </span><span style="background-color: #666666">    </span><span style="background-color: #606060">  </span><span style="background-color: #595959">  </span><span style="background-color: #5E5E5E">  </span><span style="background-color: #787878">  </span><span style="background-color: #9E9E9E">  </span><span style="background-color: #797979">  </span><span style="background-color: #5E5E5E">  </span><span style="background-color: #5F5F5F">      </span><span style="background-color: #575757">  </span><span style="background-color: #343434">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #191919">  </span><span>            </span>
<span style="background-color: #2B2B2B">  </span><span style="background-color: #3B3B3B">  </span><span style="background-color: #575757">  </span><span style="background-color: #646464">  </span><span style="background-color: #5F5F5F">  </span><span style="background-color: #5E5E5E">  </span><span style="background-color: #575757">  </span><span style="background-color: #3D3D3D">  </span><span style="background-color: #525252">  </span><span style="background-color: #656565">  </span><span style="background-color: #666666">        </span><span style="background-color: #656565">  </span><span style="background-color: #616161">  </span><span style="background-color: #595959">  </span><span style="background-color: #4B4B4B">  </span><span style="background-color: #454545">  </span><span style="background-color: #4B4B4B">  </span><span style="background-color: #555555">  </span><span style="background-color: #5D5D5D">  </span><span style="background-color: #5F5F5F">  </span><span style="background-color: #575757">  </span><span style="background-color: #343434">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #191919">  </span><span>          </span>
<span style="background-color: #3A3A3A">  </span><span style="background-color: #292929">  </span><span style="background-color: #323232">  </span><span style="background-color: #4A4A4A">  </span><span style="background-color: #626262">  </span><span style="background-color: #666666">  </span><span style="background-color: #656565">  </span><span style="background-color: #5F5F5F">  </span><span style="background-color: #616161">  </span><span style="background-color: #5E5E5E">  </span><span style="background-color: #616161">  </span><span style="background-color: #5F5F5F">  </span><span style="background-color: #666666">            </span><span style="background-color: #626262">  </span><span style="background-color: #575757">  </span><span style="background-color: #4B4B4B">  </span><span style="background-color: #454545">  </span><span style="background-color: #4A4A4A">  </span><span style="background-color: #545454">    </span><span style="background-color: #343434">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #191919">  </span><span>        </span>
<span>    </span><span style="background-color: #252525">  </span><span style="background-color: #383838">  </span><span style="background-color: #5F5F5F">  </span><span style="background-color: #616161">  </span><span style="background-color: #5B5B5B">  </span><span style="background-color: #505050">  </span><span style="background-color: #545454">  </span><span style="background-color: #8A8A8A">  </span><span style="background-color: #C5C5C5">  </span><span style="background-color: #959595">  </span><span style="background-color: #5E5E5E">  </span><span style="background-color: #636363">  </span><span style="background-color: #666666">              </span><span style="background-color: #626262">  </span><span style="background-color: #595959">  </span><span style="background-color: #4D4D4D">  </span><span style="background-color: #454545">  </span><span style="background-color: #414141">  </span><span style="background-color: #282828">  </span><span style="background-color: #1E1E1E">  </span><span style="background-color: #1D1D1D">  </span><span>      </span>
<span>    </span><span style="background-color: #212121">  </span><span style="background-color: #2C2C2C">  </span><span style="background-color: #4F4F4F">  </span><span style="background-color: #515151">  </span><span style="background-color: #5F5F5F">  </span><span style="background-color: #898989">  </span><span style="background-color: #CDCDCD">  </span><span style="background-color: #E8E8E8">  </span><span style="background-color: #DEDEDE">  </span><span style="background-color: #D8D8D8">  </span><span style="background-color: #939393">  </span><span style="background-color: #4D4D4D">  </span><span style="background-color: #525252">  </span><span style="background-color: #5E5E5E">  </span><span style="background-color: #646464">  </span><span style="background-color: #666666">              </span><span style="background-color: #636363">  </span><span style="background-color: #5A5A5A">  </span><span style="background-color: #4A4A4A">  </span><span style="background-color: #383838">  </span><span style="background-color: #323232">  </span><span style="background-color: #2A2A2A">  </span><span style="background-color: #282828">  </span><span>  </span>
<span>    </span><span style="background-color: #272727">  </span><span style="background-color: #404040">  </span><span style="background-color: #C8C8C8">  </span><span style="background-color: #DFDFDF">  </span><span style="background-color: #F0F0F0">  </span><span style="background-color: #FDFDFD">  </span><span style="background-color: #F3F3F3">  </span><span style="background-color: #DFDFDF">  </span><span style="background-color: #DDDDDD">    </span><span style="background-color: #D7D7D7">  </span><span style="background-color: #757575">  </span><span style="background-color: #2B2B2B">  </span><span style="background-color: #333333">  </span><span style="background-color: #444444">  </span><span style="background-color: #535353">  </span><span style="background-color: #5F5F5F">  </span><span style="background-color: #646464">  </span><span style="background-color: #666666">              </span><span style="background-color: #646464">  </span><span style="background-color: #5B5B5B">  </span><span style="background-color: #4F4F4F">  </span><span style="background-color: #3A3A3A">  </span><span style="background-color: #292929">  </span>
<span>    </span><span style="background-color: #242424">  </span><span style="background-color: #4F4F4F">  </span><span style="background-color: #E7E7E7">  </span><span style="background-color: #FFFFFF">    </span><span style="background-color: #F2F2F2">  </span><span style="background-color: #DFDFDF">  </span><span style="background-color: #DDDDDD">      </span><span style="background-color: #C2C2C2">  </span><span style="background-color: #6E6E6E">  </span><span style="background-color: #434343">  </span><span style="background-color: #242424">  </span><span style="background-color: #222222">  </span><span style="background-color: #282828">  </span><span style="background-color: #343434">  </span><span style="background-color: #454545">  </span><span style="background-color: #555555">  </span><span style="background-color: #606060">  </span><span style="background-color: #656565">  </span><span style="background-color: #666666">              </span><span style="background-color: #595959">  </span><span style="background-color: #313131">  </span>
<span>    </span><span style="background-color: #222222">  </span><span style="background-color: #5F5F5F">  </span><span style="background-color: #F2F2F2">  </span><span style="background-color: #FFFFFF">  </span><span style="background-color: #F4F4F4">  </span><span style="background-color: #D7D7D7">  </span><span style="background-color: #DCDCDC">  </span><span style="background-color: #DDDDDD">    </span><span style="background-color: #D1D1D1">  </span><span style="background-color: #818181">  </span><span style="background-color: #5F5F5F">  </span><span style="background-color: #5D5D5D">  </span><span style="background-color: #434343">  </span><span style="background-color: #242424">  </span><span style="background-color: #202020">    </span><span style="background-color: #222222">  </span><span style="background-color: #282828">  </span><span style="background-color: #353535">  </span><span style="background-color: #464646">  </span><span style="background-color: #565656">  </span><span style="background-color: #606060">  </span><span style="background-color: #656565">  </span><span style="background-color: #666666">        </span><span style="background-color: #585858">  </span><span style="background-color: #333333">  </span>
<span>    </span><span style="background-color: #222222">  </span><span style="background-color: #707070">  </span><span style="background-color: #FAFAFA">    </span><span style="background-color: #D2D2D2">  </span><span style="background-color: #D9D9D9">  </span><span style="background-color: #DDDDDD">    </span><span style="background-color: #D9D9D9">  </span><span style="background-color: #979797">  </span><span style="background-color: #616161">  </span><span style="background-color: #5F5F5F">    </span><span style="background-color: #5D5D5D">  </span><span style="background-color: #434343">  </span><span style="background-color: #242424">  </span><span style="background-color: #202020">        </span><span style="background-color: #222222">  </span><span style="background-color: #292929">  </span><span style="background-color: #363636">  </span><span style="background-color: #474747">  </span><span style="background-color: #575757">  </span><span style="background-color: #606060">  </span><span style="background-color: #616161">  </span><span style="background-color: #575757">  </span><span style="background-color: #404040">  </span><span style="background-color: #2B2B2B">  </span>
<span>    </span><span style="background-color: #212121">  </span><span style="background-color: #858585">  </span><span style="background-color: #FCFCFC">  </span><span style="background-color: #D9D9D9">  </span><span style="background-color: #D2D2D2">  </span><span style="background-color: #DDDDDD">    </span><span style="background-color: #DCDCDC">  </span><span style="background-color: #AEAEAE">  </span><span style="background-color: #666666">  </span><span style="background-color: #5F5F5F">        </span><span style="background-color: #5D5D5D">  </span><span style="background-color: #434343">  </span><span style="background-color: #242424">  </span><span style="background-color: #202020">            </span><span style="background-color: #222222">  </span><span style="background-color: #292929">  </span><span style="background-color: #363636">  </span><span style="background-color: #3E3E3E">  </span><span style="background-color: #363636">  </span><span style="background-color: #2B2B2B">  </span><span style="background-color: #282828">  </span>
<span>    </span><span style="background-color: #222222">  </span><span style="background-color: #9B9B9B">  </span><span style="background-color: #EAEAEA">  </span><span style="background-color: #D0D0D0">  </span><span style="background-color: #DDDDDD">      </span><span style="background-color: #C3C3C3">  </span><span style="background-color: #707070">  </span><span style="background-color: #5F5F5F">            </span><span style="background-color: #5D5D5D">  </span><span style="background-color: #434343">  </span><span style="background-color: #242424">  </span><span style="background-color: #202020">              </span><span style="background-color: #212121">  </span><span style="background-color: #242424">  </span><span style="background-color: #272727">  </span><span style="background-color: #2C2C2C">  </span><span>  </span>
<span>    </span><span style="background-color: #292929">  </span><span style="background-color: #ACACAC">  </span><span style="background-color: #DDDDDD">  </span><span style="background-color: #DCDCDC">  </span><span style="background-color: #DDDDDD">    </span><span style="background-color: #D1D1D1">  </span><span style="background-color: #818181">  </span><span style="background-color: #5F5F5F">                </span><span style="background-color: #5D5D5D">  </span><span style="background-color: #434343">  </span><span style="background-color: #242424">  </span><span style="background-color: #202020">    </span><span style="background-color: #212121">  </span><span style="background-color: #222222">  </span><span style="background-color: #232323">  </span><span style="background-color: #242424">  </span><span style="background-color: #262626">  </span><span style="background-color: #2E2E2E">  </span><span>      </span>
<span>    </span><span style="background-color: #2D2D2D">  </span><span style="background-color: #A6A6A6">  </span><span style="background-color: #DDDDDD">      </span><span style="background-color: #D9D9D9">  </span><span style="background-color: #989898">  </span><span style="background-color: #616161">  </span><span style="background-color: #5F5F5F">                  </span><span style="background-color: #5D5D5D">  </span><span style="background-color: #3E3E3E">  </span><span style="background-color: #222222">  </span><span style="background-color: #242424">  </span><span style="background-color: #262626">  </span><span style="background-color: #2B2B2B">  </span><span style="background-color: #363636">  </span><span>            </span>
<span>    </span><span style="background-color: #212121">  </span><span style="background-color: #575757">  </span><span style="background-color: #BEBEBE">  </span><span style="background-color: #DDDDDD">  </span><span style="background-color: #DCDCDC">  </span><span style="background-color: #AFAFAF">  </span><span style="background-color: #666666">  </span><span style="background-color: #5F5F5F">                    </span><span style="background-color: #5B5B5B">  </span><span style="background-color: #373737">  </span><span style="background-color: #222222">  </span><span>                    </span>
<span>    </span><span style="background-color: #171717">  </span><span style="background-color: #212121">  </span><span style="background-color: #585858">  </span><span style="background-color: #BEBEBE">  </span><span style="background-color: #C3C3C3">  </span><span style="background-color: #717171">  </span><span style="background-color: #5F5F5F">                    </span><span style="background-color: #5E5E5E">  </span><span style="background-color: #424242">  </span><span style="background-color: #252525">  </span><span style="background-color: #242424">  </span><span>                    </span>
<span>      </span><span style="background-color: #171717">  </span><span style="background-color: #212121">  </span><span style="background-color: #545454">  </span><span style="background-color: #717171">  </span><span style="background-color: #5F5F5F">                      </span><span style="background-color: #4D4D4D">  </span><span style="background-color: #292929">  </span><span style="background-color: #232323">  </span><span>                      </span>
<span>        </span><span style="background-color: #171717">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #343434">  </span><span style="background-color: #565656">  </span><span style="background-color: #5F5F5F">                  </span><span style="background-color: #565656">  </span><span style="background-color: #303030">  </span><span style="background-color: #222222">  </span><span>                        </span>
<span>          </span><span style="background-color: #171717">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #343434">  </span><span style="background-color: #565656">  </span><span style="background-color: #5F5F5F">              </span><span style="background-color: #5C5C5C">  </span><span style="background-color: #393939">  </span><span style="background-color: #232323">  </span><span style="background-color: #252525">  </span><span>                        </span>
<span>            </span><span style="background-color: #171717">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #343434">  </span><span style="background-color: #565656">  </span><span style="background-color: #5F5F5F">          </span><span style="background-color: #5E5E5E">  </span><span style="background-color: #444444">  </span><span style="background-color: #252525">  </span><span style="background-color: #222222">  </span><span>                          </span>
<span>              </span><span style="background-color: #171717">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #343434">  </span><span style="background-color: #565656">  </span><span style="background-color: #5F5F5F">        </span><span style="background-color: #4F4F4F">  </span><span style="background-color: #2A2A2A">  </span><span style="background-color: #222222">  </span><span>                            </span>
<span>                </span><span style="background-color: #171717">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #343434">  </span><span style="background-color: #565656">  </span><span style="background-color: #5F5F5F">    </span><span style="background-color: #575757">  </span><span style="background-color: #323232">  </span><span style="background-color: #222222">  </span><span>                              </span>
<span>                  </span><span style="background-color: #171717">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #343434">  </span><span style="background-color: #565656">  </span><span style="background-color: #5C5C5C">  </span><span style="background-color: #3C3C3C">  </span><span style="background-color: #232323">  </span><span style="background-color: #252525">  </span><span>                              </span>
<span>                    </span><span style="background-color: #171717">  </span><span style="background-color: #1F1F1F">  </span><span style="background-color: #343434">  </span><span style="background-color: #404040">  </span><span style="background-color: #262626">  </span><span style="background-color: #232323">  </span><span>                                </span>
<span>                      </span><span style="background-color: #171717">  </span><span style="background-color: #1E1E1E">  </span><span style="background-color: #222222">    </span><span>                                  </span>
</pre>